#include "forms.h"
#include "xoskope.h"

#include "toscope.h"


//************************************************************************

enum OSCOPE_OPERATION_MODE {
			    OSCOPE_OPERATION_MODE_NORMAL=0,
			    OSCOPE_OPERATION_MODE_ADD=1,
			    OSCOPE_OPERATION_MODE_XY=2
			   };


struct oscope_common_info
{
 float T_pos;
 float T_scale;
 float trig_level;

 float rate;
 int op_mode;
 OSCOPE_TRIG_TYPE trig_mode;
 int trig_chan;
};

struct oscope_channel_info
{
 int on;
 FL_COLOR color;
 float V_pos;
 float V_scale;
 int inv;
 int skip;

 float min,max,avr,rms;

};


//************************************************************************
//************************************************************************

extern FD_xoscope *xoscope;
extern FD_about *about;

int init_oscope_dev(oscope_common_info *ci);
void set_oscope_time_axis(oscope_common_info *ci);
void set_oscope_time_rate(oscope_common_info *ci);
int xyplot_display_handler(FL_OBJECT *,int,FL_Coord,FL_Coord,int,void *);

//**********************************

oscope_dev *osc_dev=NULL;

oscope_common_info common_info={0.0,1.0,0.0,22000,
		OSCOPE_OPERATION_MODE_NORMAL,OSCOPE_ASC_TRIG,0};

oscope_channel_info channel_info[2]={{1,FL_RED,0.0,1.0,0,0},{1,FL_GREEN,0.0,1.0,0,0}};

int run=1;
int force_refresh=0;


//************************************************************************
//************************************************************************
//************************************************************************


int init_xoskope(oscope_dev *od)
 {
   if (od==NULL) return 0;

   osc_dev=od;
   if (init_oscope_dev(&common_info)==0) return 0;

   //*************************

   fl_set_xyplot_xgrid(xoscope->xyplot_display, FL_GRID_MINOR);
   fl_set_xyplot_ygrid(xoscope->xyplot_display, FL_GRID_MINOR);

   fl_set_xyplot_fixed_xaxis(xoscope->xyplot_display,NULL," ");
   //fl_set_xyplot_fixed_xaxis(xoscope->xyplot_display,"m",NULL);

   set_oscope_time_rate(&common_info);
   fl_set_xyplot_ybounds(xoscope->xyplot_display,-1.0,1.0);

   fl_set_object_posthandler(xoscope->xyplot_display,xyplot_display_handler);
   //fl_set_xyplot_inspect(xoscope->xyplot_display,1);

   fl_set_counter_return(xoscope->counter_Vpos1,FL_RETURN_CHANGED);
   fl_set_counter_return(xoscope->counter_Vscale1,FL_RETURN_CHANGED);
   fl_set_counter_return(xoscope->counter_Vpos2,FL_RETURN_CHANGED);
   fl_set_counter_return(xoscope->counter_Vscale2,FL_RETURN_CHANGED);
   fl_set_counter_return(xoscope->counter_Tpos,FL_RETURN_CHANGED);
   fl_set_counter_return(xoscope->counter_Tscale,FL_RETURN_CHANGED);
   fl_set_counter_return(xoscope->counter_trig_level,FL_RETURN_CHANGED);

   return 1;
 }


//************************************************************************

int init_oscope_dev(oscope_common_info *ci)
 {

   if (osc_dev->open_dev()==FALSE)
     {
        printf("No se pudo abrir el dispositivo /dev/dsp \n");
        return 0;
     }
   osc_dev->reset_dev();

   osc_dev->set_rate(ci->rate);
   osc_dev->set_trig_mode(ci->trig_mode);
   osc_dev->set_trig_level(ci->trig_level);
   osc_dev->set_trig_chan(ci->trig_chan);

   printf("\nCHAN= %d\n",osc_dev->get_chan());
   printf("BITS= %d\n",osc_dev->get_bps());
   printf("RATE= %d\n\n",osc_dev->get_rate());

   return 1;
 }


void reset_oscope_dev()
 {
   osc_dev->close_dev();
   init_oscope_dev(&common_info);
 }


void set_oscope_time_rate(oscope_common_info *ci)
{
  common_info.T_scale=osc_dev->get_dt()*osc_dev->get_scanline_size();
  fl_set_counter_value(xoscope->counter_Tscale,common_info.T_scale/8);
  set_oscope_time_axis(ci);
}


void set_oscope_time_axis(oscope_common_info *ci)
{
  fl_set_xyplot_xbounds(xoscope->xyplot_display,-ci->T_scale/2,ci->T_scale/2);
}


//**********************************


void calculate_channel_info(oscope_dev *osc, int chan, oscope_channel_info *och)
 {
   int cont;
   float s;

   och->max=-1000000;
   och->min=1000000;
   och->avr=0;
   for (cont=0;cont<osc->get_scanline_size();cont++)
    {
      s=osc->get_sample(chan,cont);
      if (s<och->min) och->min=s;
      if (s>och->max) och->max=s;
      och->avr+=s;
    }
   och->avr/=osc->get_scanline_size();
}


void refresh_channel_info()
{
  char cad[16];

  calculate_channel_info(osc_dev,0,&channel_info[0]);
  calculate_channel_info(osc_dev,1,&channel_info[1]);

  //*********** Channel 1 ******************
  sprintf(cad,"%.2f",channel_info[0].min);
  fl_set_object_label(xoscope->text_min_chan1,cad);

  sprintf(cad,"%.2f",channel_info[0].max);
  fl_set_object_label(xoscope->text_max_chan1,cad);

  sprintf(cad,"%.2f",channel_info[0].avr);
  fl_set_object_label(xoscope->text_avr_chan1,cad);

  sprintf(cad,"%.2f",channel_info[0].rms);
  fl_set_object_label(xoscope->text_rms_chan1,cad);

  //*********** Channel 2 ******************
  sprintf(cad,"%.2f",channel_info[1].min);
  fl_set_object_label(xoscope->text_min_chan2,cad);

  sprintf(cad,"%.2f",channel_info[1].max);
  fl_set_object_label(xoscope->text_max_chan2,cad);

  sprintf(cad,"%.2f",channel_info[1].avr);
  fl_set_object_label(xoscope->text_avr_chan2,cad);

  sprintf(cad,"%.2f",channel_info[1].rms);
  fl_set_object_label(xoscope->text_rms_chan2,cad);

}


void refresh_mouse_info(float wx, float wy)
{
  char cad[16];

  sprintf(cad,"%.3f",wx);
  fl_set_object_label(xoscope->text_mouse_T,cad);

  sprintf(cad,"%.3f",wy*channel_info[0].V_scale);
  fl_set_object_label(xoscope->text_mouse_V1,cad);

  sprintf(cad,"%.3f",wy*channel_info[1].V_scale);
  fl_set_object_label(xoscope->text_mouse_V2,cad);

}


//**********************************


int plot_data(FL_OBJECT *xyplot, oscope_dev *osc, int chan, oscope_channel_info *och)
{
  if (osc==NULL || och==NULL || och->skip<0 || och->on==0) return 0;

  int pos,cont,inc=och->skip+1;
  int nn=osc->get_scanline_size();
  int n=nn/(och->skip+1);
  float sc=och->V_scale;

  float *dx=new float[nn];
  float *dy=new float[nn];

  if (och->inv==1) sc*=-1;

  for (pos=0,cont=0;cont<n;pos+=inc,cont++)
   {
     dx[cont]=common_info.T_pos-common_info.T_scale/2+cont*osc->get_dt();
     dy[cont]=sc*(och->V_pos+osc->get_sample(chan,pos));
     //dy[cont]=och->V_scale*(och->V_pos+sin(0.1*cont));
   }

  chan++;
  if (chan<=0) chan=1;
  fl_add_xyplot_overlay(xyplot,chan,dx,dy,n,och->color);

  delete dx,dy;

  return 1;
}


int plot_data_extra(FL_OBJECT *xyplot, oscope_dev *osc,
		    oscope_channel_info *och1, oscope_channel_info *och2, int op_mode)
{
  if (osc==NULL || och1==NULL || och1->on==0 || och2==NULL || och2->on==0) return 0;

  int cont;
  int n=osc->get_scanline_size();

  float sc1=och1->V_scale;
  float sc2=och2->V_scale;

  float *dx=new float[n];
  float *dy=new float[n];

  if (och1->inv==1) sc1*=-1;
  if (och2->inv==1) sc2*=-1;

  if (op_mode==OSCOPE_OPERATION_MODE_ADD)
   {
     for (cont=0;cont<n;cont++)
     {
      dx[cont]=common_info.T_pos-common_info.T_scale/2+cont*osc->get_dt();
      dy[cont]=sc1*(och1->V_pos+osc->get_sample(0,cont))+
	       sc2*(och2->V_pos+osc->get_sample(1,cont));

     }
   }
  else if (op_mode==OSCOPE_OPERATION_MODE_XY)
   {
     for (cont=0;cont<n;cont++)
     {
      dx[cont]=sc1*(och1->V_pos+osc->get_sample(0,cont));
      dy[cont]=sc2*(och2->V_pos+osc->get_sample(1,cont));
     }
   }

  fl_add_xyplot_overlay(xyplot,1,dx,dy,n,och1->color);

  delete dx,dy;

  return 1;
}


void prepare_xyplot(FL_OBJECT *xyplot)
{
 float dx[2]={0,1},dy[2]={0,0};
 fl_set_xyplot_data(xyplot,dx,dy,2,""," "," ");
}


void refresh_display()
{
 prepare_xyplot(xoscope->xyplot_display);

 if (common_info.op_mode==OSCOPE_OPERATION_MODE_NORMAL)
  {
   plot_data(xoscope->xyplot_display,osc_dev,0,&channel_info[0]);
   plot_data(xoscope->xyplot_display,osc_dev,1,&channel_info[1]);
  }
 else plot_data_extra(xoscope->xyplot_display,osc_dev,
		      &channel_info[0],&channel_info[1],common_info.op_mode);

}


int refresh_oscope()
{
 boolean triggered=FALSE;

 if (run==1) triggered=osc_dev->read_frame();

 if (!triggered && !force_refresh) return 0;   // Si no hay disparo pasar de largo ...

 fl_freeze_form(xoscope->xoscope);
 refresh_channel_info();
 refresh_display();
 fl_unfreeze_form(xoscope->xoscope);

 force_refresh=0;

 return 1;
}


//************************************************************************


/* callbacks and freeobj handles for form xoscope */
void CB_xyplot_display(FL_OBJECT *ob, long data)
{
  float x,y;
  int i;
  fl_get_xyplot(ob,&x,&y,&i);

  printf("MOUSE CLICK ==> T=%.4f, V=%.4f\n",x,y);

  /* fill-in code for callback */
}

void CB_button_close(FL_OBJECT *ob, long data)
{
  exit(0);
  /* fill-in code for callback */
}

void CB_button_spectrum(FL_OBJECT *ob, long data)
{
  /* fill-in code for callback */
}

void CB_button_display_setup(FL_OBJECT *ob, long data)
{
  /* fill-in code for callback */
}


void CB_logotipo(FL_OBJECT *ob, long data)
{
  /* fill-in code for callback */

   fl_show_form(about->about,FL_PLACE_CENTERFREE,FL_FULLBORDER,"About...");
}


void CB_button_run(FL_OBJECT *ob, long data)
{
  /* fill-in code for callback */
  int b=fl_get_button(ob);

  if (b==1)
   {
     printf("Run\n");
     run=1;
     fl_set_timer(xoscope->timer_refresh,0.01);
     fl_set_object_lcol(ob,FL_RED);
     fl_set_object_label(ob,"STOP");
   }
  else
   {
     printf("Stop\n");
     run=0;
     fl_set_timer(xoscope->timer_refresh,0.0);
     fl_set_object_lcol(ob,FL_GREEN);
     fl_set_object_label(ob,"RUN");
   }

}


void CB_timer_refresh(FL_OBJECT *ob, long data)
{
  refresh_oscope();
  fl_set_timer(xoscope->timer_refresh,0.01);
}



int xyplot_display_handler(FL_OBJECT *ob,int event,FL_Coord mx,FL_Coord my,int key,void *xev)
{
 //static int p=1;

 switch(event)
  {
    case FL_MOUSE:
                float wx,wy;
		mx-=ob->x;my-=ob->y;
		fl_xyplot_s2w(ob,mx,my,&wx,&wy);
                //printf("MOUSE SCREEN ==> T=%d, V=%d\n",mx,my);
                //printf("MOUSE XYPLOT ==> T=%.3f, V=%.3f\n",wx,wy);
                refresh_mouse_info(wx,wy);
		break;
/*
    case FL_PUSH:
		if (fl_mouse_button()==1) p=1;
		else p=-1;
		break;
*/

  }

 return 0;

}


//**********************************


void CB_counter_Vpos1(FL_OBJECT *ob, long)
{
  channel_info[0].V_pos=fl_get_counter_value(ob);

  force_refresh=1;
}

void CB_counter_Vscale1(FL_OBJECT *ob, long)
{
  channel_info[0].V_scale=fl_get_counter_value(ob);

  force_refresh=1;
}

void CB_check_chan1(FL_OBJECT *ob, long)
{
  channel_info[0].on=fl_get_button(ob);

  force_refresh=1;
}

void CB_button_color_chan1(FL_OBJECT *ob, long)
{
  channel_info[0].color=fl_show_colormap(channel_info[0].color);
  fl_set_object_color(ob,channel_info[0].color,FL_COL1);

  force_refresh=1;
}

void CB_button_inv_chan1(FL_OBJECT *ob, long)
{
  channel_info[0].inv=fl_get_button(ob);

  force_refresh=1;
}


void CB_counter_Vpos2(FL_OBJECT *ob, long)
{
  channel_info[1].V_pos=fl_get_counter_value(ob);

  force_refresh=1;
}

void CB_counter_Vscale2(FL_OBJECT *ob, long)
{
  channel_info[1].V_scale=fl_get_counter_value(ob);

  force_refresh=1;
}

void CB_check_chan2(FL_OBJECT *ob, long)
{
  channel_info[1].on=fl_get_button(ob);

  force_refresh=1;
}

void CB_button_color_chan2(FL_OBJECT *ob, long)
{
  channel_info[1].color=fl_show_colormap(channel_info[1].color);
  fl_set_object_color(ob,channel_info[1].color,FL_COL1);

  force_refresh=1;
}

void CB_button_inv_chan2(FL_OBJECT *ob, long)
{
  channel_info[1].inv=fl_get_button(ob);

  force_refresh=1;
}


//**********************************


void CB_counter_Tscale(FL_OBJECT *ob, long)
{
  float t=fl_get_counter_value(ob);
  common_info.T_scale=t*8;
  set_oscope_time_axis(&common_info);
  force_refresh=1;
}

void CB_counter_Tpos(FL_OBJECT *ob, long)
{
  common_info.T_pos=fl_get_counter_value(ob);
  set_oscope_time_axis(&common_info);
  force_refresh=1;
}


void CB_counter_trig_level(FL_OBJECT *ob, long)
{
  common_info.trig_level=fl_get_counter_value(ob);
  osc_dev->set_trig_level((int)common_info.trig_level);	// Conversion !!!
  force_refresh=1;
}


void CB_droplist_rate(FL_OBJECT *ob, long)
{
  float r;
  sscanf(fl_get_choice_text(ob),"%f",&r);
  common_info.rate=r;

  reset_oscope_dev();
  set_oscope_time_rate(&common_info);

  force_refresh=1;
}

void CB_droplist_opmode(FL_OBJECT *ob, long)
{
  common_info.op_mode=fl_get_choice(ob)-1;

  printf("OP MODE ==> %d\n",common_info.op_mode);

  force_refresh=1;
}

void CB_droplist_trigmode(FL_OBJECT *ob, long)
{
  common_info.trig_mode=(OSCOPE_TRIG_TYPE)(fl_get_choice(ob)-1);
  osc_dev->set_trig_mode(common_info.trig_mode);

  printf("TRIG MODE ==> %d\n",common_info.trig_mode);

  force_refresh=1;
}

void CB_droplist_trigchan(FL_OBJECT *ob, long)
{
  common_info.trig_chan=fl_get_choice(ob)-1;
  osc_dev->set_trig_chan(common_info.trig_chan);

  printf("TRIG CHAN ==> %d\n",common_info.trig_chan);

  force_refresh=1;
}


//************************************************************************

void CB_button_about_OK(FL_OBJECT *ob, long)
{
  fl_hide_form(about->about);
}

//************************************************************************
//************************************************************************

